/* ***************************************************************************** 
  This file is part of the kde-game 'kjumpingCube'

  Copyright (C) 1998 by Matthias Kiefer <Matthias.Kiefer@stud.uni-karlsruhe.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


  *******************************************************************************/  

#ifndef CUBEBOX_H
#define CUBEBOX_H

#include <qpixmap.h>
#include "cube.h"
#include "cubesim.h"
#include "engine.h"

class QPalette;
class KConfig;

/**
*@internal
*/
struct Loop
{
   int row;
   int column;
   bool finished;
};
/**
* @short Class for managing the cubes
*/		

class CubeBox : public QWidget
{
  Q_OBJECT
public:
  /**         
  * constructs a cubebox with 'dim'x'dim' cubes
  *                
  * @param dim     Size of cubebox
  * @param zoom    Size of the cubes
  * @param color1  Color of cubes, that belong to player one
  * @param color2  Color of cubes, that belong to player two
  */
  CubeBox(int dim,int zoom, QPalette *color1, QPalette *color2,QWidget *parent=0,
		 const char *name=0);
  /**
  */
  ~CubeBox(); 
  
  /**         
  * sets Zoom of the cubes,
  * the given parameter is multiplied with 20%. (5 -> 100%) 
  */
  void setZoom(int); 
  
  /**         
  * zooms in 10% 
  *
  * @return current zoomfactor        
  */
  int zoomIn(); 
  
  /**         
  * zooms out 10% 
  *
  * @return current zoomfactor        
  */
  int zoomOut();
  
  /**
  *  @return TRUE if maximum zoom  is reached (8 -> 160%)
  */
  bool maxZoom(); 
  
  /**
  * @return TRUE if minium zoom is reached (3 -> 60%)
  */
  bool minZoom();
  
  /**
  * @return current zoomfactor (divided through 20%)
  */
  int zoom();
  
  /**
  *
  * @return size of cubebox
  */
  int dim();
  
  /**
  * set colors that are used to show owners of the cubes
  *
  * @param color1  color for player one
  * @param color2  color for player two
  */
  void setColors(QPalette *color1,QPalette *color2);
  
  /**
  * sets skill of computerplayer and 'getHint()'
  * 
  * @return previous level
  */
  int setSkill(int level);
  
  /**
  * @ return current skill
  */
  int skill();
  /**
  * sets player 'player' as computer or human
  *
  * @param player 
  * @param flag: TRUE for computer, FALSE for human
  */
  void setComputerplayer(int player,bool flag);

  /**
  * returns TRUE if player 'player' is a computerPlayer
  */
  bool isComputer(int player);
  
  /**
  * returns TRUE if CubeBox is doing a move or getting a hint
  */
  bool isActive();  
  
  /**
  * stops all activities like getting a hint or doing a move
  */
  void stopActivities();
  
  /**
  */
  void saveGame(KConfig *);
  
  /**
  */
  void restoreGame(KConfig *);
  
  /**
  * undo last move
  *
  * if fromSessionManagment=TRUE than no signal playerChanged(int) will be sent
  * @return TRUE if successful
  */
  bool undo(bool fromSessionManagment=FALSE);
  
  /**
  * reset cubebox for a new game
  */
  void reset();
  
  /**
  * computes a possibility to move and shows it by highlightning
  * this cube
  */
  void getHint();

  /**
  * stops process started by getHint().
  * @see CubeBox#getHint()
  */
  void stopThinking();  
  
public slots:
  /**
  */
  void showWinner(int winner);

  /**
  * checks if cube at ['row','column'] is clickable by the current player.
  * if true, it increases this cube and checks the playingfield  
  */
  void checkClick(int row,int column);

  /**
  * checks if 'player' is a computerplayer an computes next move if TRUE
  */
  void checkComputerplayer(int player);

      
signals:
  /**
  * emitted if move has finished and it's now the other players turn 
  */
  void playerChanged(int newPlayer);

  /**
  * emitted if a color for a player has changed
  */
  void colorChanged();
  
  /**
  * emitted if the computation of the hint was finished
  */
  void stoppedThinking();
  
  /**
  * emitted if the computation of the hint was started
  */
  void startedThinking();
  
  
protected:
  /**
  */   
  void resizeEvent(QResizeEvent *); 

  /**
  * do the next step of the loop for checking the playingfield
  */
  void timerEvent(QTimerEvent *);
  
private:
  /**
  * array of Cube
  */
  Cube ***cubes;

  /**
  * size of cubebox
  */
  int cubeDim;
  /**
  * size of cubes
  * you can change this with 
  * @see CubeBox#zoomIn()
  * @see CubeBox#zoomOut()
  * @see CubeBox#setZoom() 
  */
  int cubeSize;
  /**
  * player, who is on turn
  */
  int actualPlayer;
  
  /**
  * this holds information about the playingfield before the last move
  * @see CubeSimBox
  */
  CubeSimBox previousField;
  
  /**
  * This is the brain of the game
  * @see Engine
  */
  Engine think;
  
  /**
  * which player is played by the computer
  */
  bool computerPlayer[2];
  
  /**
  * flag if cubebox checks the playingfield at the moment
  */
  bool doingMove;

  /**
  * flag if CubeBox has to stop activities()
  */
  bool stopped;
  
  /**
  */     
  Loop loop;
  
  /**
  * increases the cube at row 'row' and column 'column' ,
  * and starts the Loop for checking the playingfield 
  */
  void doMove(int row,int column);
  
  /**
  *
  */
  void startLoop();
  
  /**
  *
  */
  void stopLoop();
  
  /**
  * do the next step of the loop for checking the playingfield
  */ 
  void nextLoopstep();
  
  /**
  * increases the neighbours of cube at ['row','column']
  */
  void increaseNeighbours(int row,int column);
 
  
  
  friend class CubeSimBox;  
};

#endif // CUBEBOX_H
